import ij.*;
import ij.plugin.filter.PlugInFilter;
import ij.plugin.filter.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.measure.*;
import java.awt.event.*;

/** This plugin calculates surface roughness statistics based on  a topographical images.
    The calculated roughness values are:

    Ra:     	Arithmetical mean deviation
    Rq:     	Root mean square deviation
    Rku:    	Kurtosis of the assessed profile
    Rsk:    	Skewness of the assessed profile
    Rv  		Lowest valley (given by the min measurements)
    Rp  		Highest peak (given by the max measurements)
    Rt: 		The total height of the profile

    Gary Chinga 020501

    History:		030221 - 	new option to divide the image into subimages and calculate the roughness
								statistics in each subimage. The results is given as the average of all
								the subimages
					030221		Get the image calibration automatically

    Last revised 	030221
*/

public class Roughness_calculation implements PlugInFilter {
    static int pixelSize = 1;
    static int sampleSize = 4;
    ImagePlus imp;
    boolean canceled = false;
    boolean whole = false;
    ResultsTable rt = new ResultsTable();
    int slice,ms,counter,w,h,c;
	double ps;

    public int setup(String arg, ImagePlus imp) {
        this.imp = imp;
        return DOES_ALL+NO_UNDO;
    }

    public void run(ImageProcessor ip) {
        	float temp = 0;
        	w = ip.getWidth();
        	h = ip.getHeight();
			if (h<w) c=h; else c=w;		//Calculates the smallest dimension
            int nSlices = imp.getStackSize();

        	Calibration cal = imp.getCalibration();
        	ps = cal.pixelWidth;

        	getDetails();
        	if (canceled) return;

			//Calibrate pixel unit
			ms=(int) (ms/ps);

			if (whole) ms = c;

        	int nXmasks = w/ms;
        	int nYmasks = h/ms;

       		for (int n = 0; n < nSlices; n++){
				IJ.showProgress((double)n/nSlices);
				IJ.showStatus("a: "+n+"/"+nSlices);

				imp.setSlice(n+1);
				counter=0;
				double Ra=0, Rq=0, Rsk=0, Rku=0, Rv=0, Rp=0, Rt=0;
        		for (int i = 0;i<nYmasks;i++){
					int yStep=i*ms;

        			for (int ii = 0;ii<nXmasks;ii++){
					int xStep=ii*ms;
        			float min = Float.MAX_VALUE;
        			float max = -Float.MAX_VALUE;
        			float ra = 0, rq = 0, sk = 0, ku = 0;

					//Accumulates stats in each subimages
        			for (int y = yStep; y < (yStep+ms); y++){
            				for (int x = xStep; x < (xStep+ms); x++){
                					int index = x + ms*y;
                					temp = ip.getPixelValue(x,y);
                					rq = rq + (temp*temp);
                					sk = sk + (temp*temp*temp);
                					ku = ku + (temp*temp*temp*temp);
                					ra = ra +  Math.abs(temp);
                					if (temp<min) min = temp;
               					if (temp>max) max = temp;
            				}
        			}
       				// Calculate Roughness stats of the subimage(Ra, Rq, Rsk, Rku)
        			double Pa = ra/(ms*ms);
        			double Pq = Math.sqrt(rq/(ms*ms));
        			double Psk = sk / ((ms*ms)*(Pq*Pq*Pq));
        			double Pku = ku / ((ms*ms)*(Pq*Pq*Pq*Pq));

					//sums the statistics of each subimage
					counter++;
					Ra = Ra+Pa;
					Rq = Rq+Pq;
					Rsk=Rsk+Psk;
					Rku = Rku+Pku;
        			//Assign the values of min and max to the variables Rp and Rv
        			Rv = Rv+min;
        			Rp = Rp+max;
				}

			}

			//Calculates the average of all the subimages for each image
			Ra = Ra/(nXmasks*nYmasks);
			Rq = Rq/(nXmasks*nYmasks);
			Rsk=Rsk/(nXmasks*nYmasks);
			Rku = Rku/(nXmasks*nYmasks);
        	//Assign the values of min and max to the variables Rp and Rv
        	Rv = Rv/(nXmasks*nYmasks);
        	Rp = Rp/(nXmasks*nYmasks);
			Rt = Rt + (Rp + Math.abs(Rv));

        	//Write the results in the result table
        	rt.incrementCounter();
        	rt.addValue("#Subimages", counter);
        	rt.addValue("Rq", Rq);
        	rt.addValue("Ra", Ra);
        	rt.addValue("Rsk", Rsk);
        	rt.addValue("Rku", Rku);
        	rt.addValue("Rp", Rp);
        	rt.addValue("Rv", Rv);
        	rt.addValue("Rt", Rt);
        	if (n==0)
            		IJ.setColumnHeadings(rt.getColumnHeadings());
       		IJ.write(rt.getRowAsString(rt.getCounter()-1));
      	}

		IJ.showProgress(1.0);

}

	void getDetails() {

       	GenericDialog gd = new GenericDialog("Surface statistics details...");
   		String units = imp.getCalibration().getUnits();

    	gd.addNumericField("Min sampling length ("+units+"):  ",(ps*4), 0);

		if (ps==1)
			units = imp.getCalibration().getUnit();
		int decimalPlaces = (int)ps==ps?0:2;
		if (!units.startsWith("pixel"))
 			gd.addMessage("        (Pixel size = "+IJ.d2s(ps,decimalPlaces)+" "+units+")");
		gd.addCheckbox("Measure on whole image ", true);

		gd.showDialog();
		if (gd.wasCanceled()) {
		  	canceled = true;
			return;
		}

		ms= (int) gd.getNextNumber();
		whole = gd.getNextBoolean();

		if (ms>(c*ps) || ms<(4*ps)) {
			canceled = true;
			IJ.showMessage("Min and Max recommened radius are "+(4*ps)+" and "+(c*ps)+" respectively");
			return;
		}
	}
}



